@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
 
 package IO::Async::FileStream;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.69';
+our $VERSION = '0.72';
 
 use base qw( IO::Async::Stream );
 
@@ -164,7 +164,9 @@ sub configure
       $params{read_handle} = $self->{file}->handle;
    }
    elsif( exists $params{handle} or exists $params{read_handle} ) {
-      my $handle = delete $params{handle} // delete $params{read_handle};
+      my $handle = delete $params{handle};
+      defined $handle or $handle = delete $params{read_handle};
+
       $self->{file}->configure( handle => $handle );
       $params{read_handle} = $self->{file}->handle;
    }
@@ -303,7 +305,7 @@ sub seek
    $success = $filestream->seek_to_last( $str_pattern, %opts )
 
 Callable only during the C<on_initial> event. Attempts to move the read
-position in the filehandle to just after the last occurance of a given match.
+position in the filehandle to just after the last occurrence of a given match.
 C<$str_pattern> may be a literal string or regexp pattern. 
 
 Returns a true value if the seek was successful, or false if not. Takes the
