@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2018 -- leonerd@leonerd.org.uk
 
 package IO::Async::Process;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.69';
+our $VERSION = '0.72';
 
 use Carp;
 
@@ -55,6 +55,28 @@ C<IO::Async::Process> - start and manage a child process
 
  $loop->run;
 
+Also accessible via the L<IO::Async::Loop/open_process> method:
+
+ $loop->open_process(
+    command => [ "/bin/ping", "-c4", "some.host" ],
+
+    stdout => {
+       on_read => sub {
+          my ( $stream, $buffref, $eof ) = @_;
+          while( $$buffref =~ s/^(.*)\n// ) {
+             print "PING wrote: $1\n";
+          }
+          return 0;
+       },
+    },
+
+    on_finish => sub {
+       my ( $pid, $exitcode ) = @_;
+       my $status = ( $exitcode >> 8 );
+       ...
+    },
+ );
+
 =head1 DESCRIPTION
 
 This subclass of L<IO::Async::Notifier> starts a child process, and invokes a
@@ -82,7 +104,7 @@ number and string values. After a successful C<exec()> call, this condition
 can no longer happen.
 
 Note that this has a different name and a different argument order from
-C<< Loop->open_child >>'s C<on_error>.
+C<< Loop->open_process >>'s C<on_error>.
 
 If this is not provided and the process exits with an exception, then
 C<on_finish> is invoked instead, being passed just the exit code.
@@ -232,6 +254,17 @@ The child will be given the reading end of a pipe. The string given by the
 C<from> parameter will be written to the child. When all of the data has been
 written the pipe will be closed.
 
+=item prefork => CODE
+
+Only valid for handles with a C<via> of C<socketpair>. The code block runs
+after the C<socketpair(2)> is created, but before the child is forked. This
+is handy for when you adjust both ends of the created socket (for example, to
+use C<setsockopt(3)>) from the controlling parent, before the child code runs.
+The arguments passed in are the L<IO::Socket> objects for the parent and child
+ends of the socket.
+
+ $prefork->( $localfd, $childfd )
+
 =back
 
 =head2 stdin => ...
@@ -355,7 +388,7 @@ sub configure_fd
    }
 
    if( defined $via and $via == FD_VIA_SOCKETPAIR ) {
-      $self->{fd_opts}{$fd}{$_} = delete $args{$_} for qw( family socktype );
+      $self->{fd_opts}{$fd}{$_} = delete $args{$_} for qw( family socktype prefork );
    }
 
    keys %args and croak "Unexpected extra keys for fd $fd - " . join ", ", keys %args;
@@ -438,6 +471,8 @@ sub _prepare_fds
       elsif( $via == FD_VIA_SOCKETPAIR ) {
          my ( $myfd, $childfd ) = IO::Async::OS->socketpair( $opts->{family}, $opts->{socktype} ) or croak "Unable to socketpair() - $!";
 
+         $opts->{prefork}->( $myfd, $childfd ) if $opts->{prefork};
+
          $handle->configure( handle => $myfd );
 
          if( $key eq "stdio" ) {
@@ -866,6 +901,32 @@ using the C<pipe_write> value for C<via>:
 
  $process->stdin->write( "Here is some more data\n" );
 
+=head2 Setting socket options
+
+By using the C<prefork> code block you can change the socket receive buffer
+size at both ends of the socket before the child is forked (at which point it
+would be too late for the parent to be able to change the child end of the
+socket).
+
+ use Socket qw( SOL_SOCKET SO_RCVBUF );
+
+ my $process = IO::Async::Process->new(
+    command => [ "command-to-read-from-and-write-to", "arguments" ],
+    stdio => {
+       via => "socketpair",
+       prefork => sub {
+          my ( $parentfd, $childfd ) = @_;
+
+          # Set parent end of socket receive buffer to 3 MB
+          $parentfd->setsockopt(SOL_SOCKET, SO_RCVBUF, 3 * 1024 * 1024);
+          # Set child end of socket receive buffer to 3 MB
+          $childfd ->setsockopt(SOL_SOCKET, SO_RCVBUF, 3 * 1024 * 1024);
+       },
+    },
+ );
+
+ $loop->add( $process );
+
 =cut
 
 =head1 AUTHOR
